using System;
using System.Text;
//using org.bouncycastle.math;
//using org.bouncycastle.math.ec;
//using org.bouncycastle.security;
using org.bouncycastle.crypto;
//using org.bouncycastle.crypto.signers;
using org.bouncycastle.crypto.digests;
//using org.bouncycastle.crypto.engines;
//using org.bouncycastle.crypto.agreement;
//using org.bouncycastle.crypto.generators;
//using org.bouncycastle.crypto.parameters;
//using org.bouncycastle.crypto.macs;
//using org.bouncycastle.crypto.modes;
//using org.bouncycastle.crypto.paddings;
using org.bouncycastle.util.encoders;
using org.bouncycastle.util.test;


namespace test.crypto
{

	/**
	 * standard vector test for MD5 from "Handbook of Applied Cryptography", page 345.
	 */
	public class MD5DigestTest: Test
	{
		static private String  testVec1 = "";
		static private String  resVec1 = "d41d8cd98f00b204e9800998ecf8427e";

		static private String  testVec2 = "61";
		static private String  resVec2 = "0cc175b9c0f1b6a831c399e269772661";

		static private String  testVec3 = "616263";
		static private String  resVec3 = "900150983cd24fb0d6963f7d28e17f72";

		static private String  testVec4 = "6162636465666768696a6b6c6d6e6f707172737475767778797a";
		static private String  resVec4 = "c3fcd3d76192e4007dfb496cca67e13b";

		public String getName()
		{
			return "MD5";
		}

		public TestResult perform()
		{
			Digest  digest = new MD5Digest();
			byte[]  resBuf = new byte[digest.getDigestSize()];
			String  resStr;

			//
			// test 1
			//
			digest.doFinal(resBuf, 0);

			resStr = ByteArray2String(Hex.encode(resBuf));
			if (!resVec1.Equals(resStr))
			{
				return new SimpleTestResult(false,
					"MD5 failing standard vector test 1"
					+ System.Environment.NewLine
					+ "    expected: " + resVec1
					+ System.Environment.NewLine
					+ "    got     : " + resStr);
			}

			//
			// test 2
			//
			byte[]  bytes = Hex.decode(testVec2);

			digest.update(bytes, 0, bytes.Length);

			digest.doFinal(resBuf, 0);

			resStr = ByteArray2String(Hex.encode(resBuf));
			if (!resVec2.Equals(resStr))
			{
				return new SimpleTestResult(false,
					"MD5 failing standard vector test 2"
					+ System.Environment.NewLine
					+ "    expected: " + resVec2
					+ System.Environment.NewLine
					+ "    got     : " + resStr);
			}

			//
			// test 3
			//
			bytes = Hex.decode(testVec3);

			digest.update(bytes, 0, bytes.Length);

			digest.doFinal(resBuf, 0);

			resStr = ByteArray2String(Hex.encode(resBuf));
			if (!resVec3.Equals(resStr))
			{
				return new SimpleTestResult(false,
					"MD5 failing standard vector test 3"
					+ System.Environment.NewLine
					+ "    expected: " + resVec3
					+ System.Environment.NewLine
					+ "    got     : " + resStr);
			}

			//
			// test 4
			//
			bytes = Hex.decode(testVec4);

			digest.update(bytes, 0, bytes.Length);

			digest.doFinal(resBuf, 0);

			resStr = ByteArray2String(Hex.encode(resBuf));
			if (!resVec4.Equals(resStr))
			{
				return new SimpleTestResult(false,
					"MD5 failing standard vector test 4"
					+ System.Environment.NewLine
					+ "    expected: " + resVec4
					+ System.Environment.NewLine
					+ "    got     : " + resStr);
			}

			//
			// test 5
			//
			bytes = Hex.decode(testVec4);

			digest.update(bytes, 0, bytes.Length/2);

			// clone the Digest
			Digest d = new MD5Digest((MD5Digest)digest);

			digest.update(bytes, bytes.Length/2, bytes.Length - bytes.Length/2);
			digest.doFinal(resBuf, 0);

			resStr = ByteArray2String(Hex.encode(resBuf));
			if (!resVec4.Equals(resStr))
			{
				return new SimpleTestResult(false,
					"MD5 failing standard vector test 5"
					+ System.Environment.NewLine
					+ "    expected: " + resVec4
					+ System.Environment.NewLine
					+ "    got     : " + resStr);
			}

			d.update(bytes, bytes.Length/2, bytes.Length - bytes.Length/2);
			d.doFinal(resBuf, 0);

			resStr = ByteArray2String(Hex.encode(resBuf));
			if (!resVec4.Equals(resStr))
			{
				return new SimpleTestResult(false,
					"MD5 failing standard vector test 5"
					+ System.Environment.NewLine
					+ "    expected: " + resVec4
					+ System.Environment.NewLine
					+ "    got     : " + resStr);
			}
			return new SimpleTestResult(true, getName() + ": Okay");
		}
		private String ByteArray2String(byte[] bs)
		{
			StringBuilder s = new StringBuilder();
			for (int i = 0; i < bs.Length; i++) 
			{
				s.Append(Convert.ToChar(bs[i]));
			}
			return s.ToString();
		}

		[STAThread]
		static void Main(string[] args)
		{
			MD5DigestTest   test = new MD5DigestTest();
			TestResult      result = test.perform();

			Console.WriteLine(result.ToString());
		}
	}

}
