using System;
using System.Collections;
using org.bouncycastle.asn1;
using org.bouncycastle.asn1.x509;

namespace org.bouncycastle.asn1.pkcs
{
	public class EncryptedPrivateKeyInfo: DEREncodable
	{
		private AlgorithmIdentifier algId;
		private ASN1OctetString     data;

		public EncryptedPrivateKeyInfo(
			ASN1Sequence  seq)
		{
			IEnumerator e = seq.getObjects();
			e.MoveNext();
			algId = new AlgorithmIdentifier((ASN1Sequence)e.Current);
			e.MoveNext();
			data = (ASN1OctetString)e.Current;
		}

		public EncryptedPrivateKeyInfo(
			AlgorithmIdentifier algId,
			byte[]              encoding)
		{
			this.algId = algId;
			this.data = new DEROctetString(encoding);
		}

		public AlgorithmIdentifier getEncryptionAlgorithm()
		{
			return algId;
		}

		public byte[] getEncryptedData()
		{
			return data.getOctets();
		}

		/**
		 * EncryptedPrivateKeyInfo ::= SEQUENCE {
		 *      encryptionAlgorithm AlgorithmIdentifier {{KeyEncryptionAlgorithms}},
		 *      encryptedData EncryptedData
		 * }
		 *
		 * EncryptedData ::= OCTET STRING
		 *
		 * KeyEncryptionAlgorithms ALGORITHM-IDENTIFIER ::= {
		 *          ... -- For local profiles
		 * }
		 */
		public DERObject getDERObject()
		{
			ASN1EncodableVector v = new ASN1EncodableVector();

			v.add(algId);
			v.add(data);

			return new DERSequence(v);
		}
	}

}