
using System;
using System.Collections;
using System.Text;
using org.bouncycastle.util.encoders;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.util
{
	public class ASN1Dump
	{
		private static String  TAB = "    ";

		/**
		 * dump a DER object as a formatted string with indentation
		 *
		 * @param obj the DERObject to be dumped out.
		 */
		public static String _dumpAsString(
			String      indent,
			DERObject   obj)
		{
			if (typeof(ASN1Sequence).IsInstanceOfType(obj))
			{
				StringBuilder    buf = new StringBuilder();
				IEnumerator     e = ((ASN1Sequence)obj).getObjects();
				String          tab = indent + TAB;

				buf.Append(indent);
				if (typeof(BERConstructedSequence).IsInstanceOfType(obj))
				{
					buf.Append("BER ConstructedSequence");
				}
				else if (typeof(DERConstructedSequence).IsInstanceOfType(obj))
				{
					buf.Append("DER ConstructedSequence");
				}
				else if (typeof(DERSequence).IsInstanceOfType(obj))
				{
					buf.Append("DER Sequence");
				}
				else if (typeof(BERSequence).IsInstanceOfType(obj))
				{
					buf.Append("BER Sequence");
				}
				else
				{
					buf.Append("Sequence");
				}

				buf.Append(System.Environment.NewLine);

				while (e.MoveNext())
				{
					Object  o = e.Current;

					if (o == null || o.Equals(new DERNull()))
					{
						buf.Append(tab);
						buf.Append("NULL");
						buf.Append(System.Environment.NewLine);
					}
					else if (typeof(DERObject).IsInstanceOfType(o))
					{
						buf.Append(_dumpAsString(tab, (DERObject)o));
					}
					else
					{
						buf.Append(_dumpAsString(tab, ((DEREncodable)o).getDERObject()));
					}
				}
				return buf.ToString();
			}
			else if (typeof(DERTaggedObject).IsInstanceOfType(obj))
			{
				StringBuilder    buf = new StringBuilder();
				String          tab = indent + TAB;

				buf.Append(indent);
				if (typeof(BERTaggedObject).IsInstanceOfType(obj))
				{
					buf.Append("BER Tagged [");
				}
				else
				{
					buf.Append("Tagged [");
				}

				DERTaggedObject o = (DERTaggedObject)obj;

				//buf.Append(Integer.ToString(o.getTagNo()));
				buf.Append(o.getTagNo().ToString());
				buf.Append("]");

				if (!o.isExplicit())
				{
					buf.Append(" IMPLICIT ");
				}

				buf.Append(System.Environment.NewLine);

				if (o.isEmpty())
				{
					buf.Append(tab);
					buf.Append("EMPTY");
					buf.Append(System.Environment.NewLine);
				}
				else
				{
					buf.Append(_dumpAsString(tab, o.getObject()));
				}

				return buf.ToString();
			}
			else if (typeof(DERConstructedSet).IsInstanceOfType(obj))
			{
				StringBuilder    buf = new StringBuilder();
				IEnumerator     e = ((ASN1Set)obj).getObjects();
				String          tab = indent + TAB;

				buf.Append(indent);
				buf.Append("ConstructedSet");
				buf.Append(System.Environment.NewLine);

				while (e.MoveNext())
				{
					Object  o = e.Current;

					if (o == null)
					{
						buf.Append(tab);
						buf.Append("NULL");
						buf.Append(System.Environment.NewLine);
					}
					else if (typeof(DERObject).IsInstanceOfType(o))
					{
						buf.Append(_dumpAsString(tab, (DERObject)o));
					}
					else
					{
						buf.Append(_dumpAsString(tab, ((DEREncodable)o).getDERObject()));
					}
				}
				return buf.ToString();
			}
			else if (typeof(BERSet).IsInstanceOfType(obj))
			{
				StringBuilder    buf = new StringBuilder();
				IEnumerator     e = ((ASN1Set)obj).getObjects();
				String          tab = indent + TAB;

				buf.Append(indent);
				buf.Append("BER Set");
				buf.Append(System.Environment.NewLine);

				while (e.MoveNext())
				{
					Object  o = e.Current;

					if (o == null)
					{
						buf.Append(tab);
						buf.Append("NULL");
						buf.Append(System.Environment.NewLine);
					}
					else if (typeof(DERObject).IsInstanceOfType(o))
					{
						buf.Append(_dumpAsString(tab, (DERObject)o));
					}
					else
					{
						buf.Append(_dumpAsString(tab, ((DEREncodable)o).getDERObject()));
					}
				}
				return buf.ToString();
			}
			else if (typeof(DERSet).IsInstanceOfType(obj))
			{
				StringBuilder    buf = new StringBuilder();
				IEnumerator     e = ((ASN1Set)obj).getObjects();
				String          tab = indent + TAB;

				buf.Append(indent);
				buf.Append("DER Set");
				buf.Append(System.Environment.NewLine);

				while (e.MoveNext())
				{
					Object  o = e.Current;

					if (o == null)
					{
						buf.Append(tab);
						buf.Append("NULL");
						buf.Append(System.Environment.NewLine);
					}
					else if (typeof(DERObject).IsInstanceOfType(o))
					{
						buf.Append(_dumpAsString(tab, (DERObject)o));
					}
					else
					{
						buf.Append(_dumpAsString(tab, ((DEREncodable)o).getDERObject()));
					}
				}
				return buf.ToString();
			}
			else if (typeof(DERObjectIdentifier).IsInstanceOfType(obj))
			{
				return indent + "ObjectIdentifier(" + ((DERObjectIdentifier)obj).getId() + ")" + System.Environment.NewLine;
			}
			else if (typeof(DERBoolean).IsInstanceOfType(obj))
			{
				return indent + "Boolean(" + ((DERBoolean)obj).isTrue() + ")" + System.Environment.NewLine;
			}
			else if (typeof(DERInteger).IsInstanceOfType(obj))
			{
				return indent + "Integer(" + ((DERInteger)obj).getValue() + ")" + System.Environment.NewLine;
			}
			else if (typeof(DEROctetString).IsInstanceOfType(obj))
			{
				return indent + obj.ToString() + "[" + ((ASN1OctetString)obj).getOctets().Length + "] " + System.Environment.NewLine;
			}
			else if (typeof(DERIA5String).IsInstanceOfType(obj))
			{
				return indent + "IA5String(" + ((DERIA5String)obj).getString() + ") " + System.Environment.NewLine;
			}
			else if (typeof(DERPrintableString).IsInstanceOfType(obj))
			{
				return indent + "PrintableString(" + ((DERPrintableString)obj).getString() + ") " + System.Environment.NewLine;
			}
			else if (typeof(DERVisibleString).IsInstanceOfType(obj))
			{
				return indent + "VisibleString(" + ((DERVisibleString)obj).getString() + ") " + System.Environment.NewLine;
			}
			else if (typeof(DERBMPString).IsInstanceOfType(obj))
			{
				return indent + "BMPString(" + ((DERBMPString)obj).getString() + ") " + System.Environment.NewLine;
			}
			else if (typeof(DERT61String).IsInstanceOfType(obj))
			{
				return indent + "T61String(" + ((DERT61String)obj).getString() + ") " + System.Environment.NewLine;
			}
			else if (typeof(DERUTCTime).IsInstanceOfType(obj))
			{
				return indent + "UTCTime(" + ((DERUTCTime)obj).getTime() + ") " + System.Environment.NewLine;
			}
			else if (typeof(DERUnknownTag).IsInstanceOfType(obj))
			{
				byte [] dataBytes = Hex.encode(((DERUnknownTag)obj).getData());
				String s = ASN1Dump.ByteArray2String(dataBytes);
				return indent + "Unknown " + ((DERUnknownTag)obj).getTag().ToString("X") + " " + s + System.Environment.NewLine;
			}
			else
			{
				return indent + obj.ToString() + System.Environment.NewLine;
			}
		}

		/**
		 * dump out a DER object as a formatted string
		 *
		 * @param obj the DERObject to be dumped out.
		 */
		public static String dumpAsString(
			Object   obj)
		{
			if (typeof(DERObject).IsInstanceOfType(obj))
			{
				return _dumpAsString("", (DERObject)obj);
			}
			else if (typeof(DEREncodable).IsInstanceOfType(obj))
			{
				return _dumpAsString("", ((DEREncodable)obj).getDERObject());
			}

			return "unknown object type " + obj.ToString();
		}
		private static String ByteArray2String(byte[] bs)
		{
			StringBuilder s = new StringBuilder();
			for (int i = 0; i < bs.Length; i++) 
			{
				s.Append(Convert.ToChar(bs[i]));
			}
			return s.ToString();
		}
	}

}