using System;
using System.Collections;
//using org.bouncycastle.io;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.x509
{
	/**
	* <pre>
	* DigestInfo::=SEQUENCE{
	*          digestAlgorithm  AlgorithmIdentifier,
	*          digest OCTET STRING }
	* </pre>
	*/
	public class DigestInfo: DEREncodable
	{
		private byte[]                  digest;
		private AlgorithmIdentifier     algId;

		public static DigestInfo getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(ASN1Sequence.getInstance(obj, flagExplicit));
		}

		public static DigestInfo getInstance(
			Object  obj)
		{
			if (typeof(DigestInfo).IsInstanceOfType(obj))
			{
				return (DigestInfo)obj;
			}
			else if (typeof(ASN1Sequence).IsInstanceOfType(obj))
			{
				return new DigestInfo((ASN1Sequence)obj);
			}

			throw new ArgumentException("unknown object in factory");
		}

		public DigestInfo(
			AlgorithmIdentifier  algId,
			byte[]               digest)
		{
			this.digest = digest;
			this.algId = algId;
		}

		public DigestInfo(
			ASN1Sequence  obj)
		{
			IEnumerator             e = obj.getObjects();

			e.MoveNext();
			algId = AlgorithmIdentifier.getInstance(e.Current);
			e.MoveNext();
			digest = ((ASN1OctetString)e.Current).getOctets();
		}

		public AlgorithmIdentifier getAlgorithmId()
		{
			return algId;
		}

		public byte[] getDigest()
		{
			return digest;
		}

		public DERObject getDERObject()
		{
			ASN1EncodableVector  v = new ASN1EncodableVector();

			v.add(algId);
			v.add(new DEROctetString(digest));

			return new DERSequence(v);
		}
	}

}