
using System;
//using System.Collections;
//using org.bouncycastle.io;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.x509
{
	public class DistributionPoint: DEREncodable
	{
		ASN1Sequence  seq = null;

		public static DistributionPoint getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(ASN1Sequence.getInstance(obj, flagExplicit));
		}

		public static DistributionPoint getInstance(
			Object obj)
		{
			if ((obj == null) || (typeof(DistributionPoint).IsInstanceOfType(obj)))
			{
				return (DistributionPoint)obj;
			}
	        
			if (typeof(ASN1Sequence).IsInstanceOfType(obj))
			{
				return new DistributionPoint((ASN1Sequence)obj);
			}
	        
			throw new ArgumentException("Invalid DistributionPoint: " + obj.GetType().ToString());
		}

		public DistributionPoint(
			ASN1Sequence seq)
		{
			this.seq = seq;
		}
	    
		public DistributionPoint(
			DistributionPointName   distributionPoint,
			ReasonFlags             reasons,
			GeneralNames            cRLIssuer)
		{
			ASN1EncodableVector  v = new ASN1EncodableVector();

			if (distributionPoint != null)
			{
				v.add(new DERTaggedObject(0, distributionPoint));
			}

			if (reasons != null)
			{
				v.add(new DERTaggedObject(1, reasons));
			}

			if (cRLIssuer != null)
			{
				v.add(new DERTaggedObject(2, cRLIssuer));
			}

			seq = new DERSequence(v);
		}

		/**
		* <pre>
		* DistributionPoint ::= SEQUENCE {
		*      distributionPoint [0] DistributionPointName OPTIONAL,
		*      reasons           [1] ReasonFlags OPTIONAL,
		*      cRLIssuer         [2] GeneralNames OPTIONAL
		* }
		* </pre>
		*/
		public DERObject getDERObject()
		{
			return seq;
		}
	}

}