
using System;
using System.Collections;
using org.bouncycastle.math;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.x509
{
	public class RSAPublicKeyStructure: DEREncodable
	{
		private BigInteger  modulus;
		private BigInteger  publicExponent;

		public static RSAPublicKeyStructure getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(ASN1Sequence.getInstance(obj, flagExplicit));
		}

		public static RSAPublicKeyStructure getInstance(
			Object obj)
		{
			if ((obj == null) || (typeof(RSAPublicKeyStructure).IsInstanceOfType(obj)))
			{
				return (RSAPublicKeyStructure)obj;
			}
	        
			if (typeof(ASN1Sequence).IsInstanceOfType(obj))
			{
				return new RSAPublicKeyStructure((ASN1Sequence)obj);
			}
	        
			throw new ArgumentException("Invalid RSAPublicKeyStructure: " + obj.GetType().ToString());
		}
	    
		public RSAPublicKeyStructure(
			BigInteger  modulus,
			BigInteger  publicExponent)
		{
			this.modulus = modulus;
			this.publicExponent = publicExponent;
		}

		public RSAPublicKeyStructure(
			ASN1Sequence  seq)
		{
			IEnumerator e = seq.getObjects();
			e.MoveNext();
			modulus = ((DERInteger)e.Current).getPositiveValue();
			e.MoveNext();
			publicExponent = ((DERInteger)e.Current).getPositiveValue();
		}

		public BigInteger getModulus()
		{
			return modulus;
		}

		public BigInteger getPublicExponent()
		{
			return publicExponent;
		}

		/**
		* This outputs the key in PKCS1v2 format.
		* <pre>
		*      RSAPublicKey ::= SEQUENCE {
		*                          modulus INTEGER, -- n
		*                          publicExponent INTEGER, -- e
		*                      }
		* </pre>
		* <p>
		*/
		public DERObject getDERObject()
		{
			ASN1EncodableVector  v = new ASN1EncodableVector();

			v.add(new DERInteger(getModulus()));
			v.add(new DERInteger(getPublicExponent()));

			return new DERSequence(v);
		}
	}

}