using System;
using org.bouncycastle.crypto;
using org.bouncycastle.math;

namespace org.bouncycastle.crypto.parameters
{
	public class DHParameters : CipherParameters
	{
		private BigInteger              g;
		private BigInteger              p;
		private BigInteger              q;
		private int                     j;
		private DHValidationParameters  validation;

		public DHParameters(
			BigInteger  p,
			BigInteger  g)
		{
			this.g = g;
			this.p = p;
		}

		public DHParameters(
			BigInteger  p,
			BigInteger  g,
			BigInteger  q,
			int         j)
		{
			this.g = g;
			this.p = p;
			this.q = q;
			this.j = j;
		}   

		public DHParameters(
			BigInteger              p,
			BigInteger              g,
			BigInteger              q,
			int                     j,
			DHValidationParameters  validation)
		{
			this.g = g;
			this.p = p;
			this.q = q;
			this.j = j;
			this.validation = validation;
		}   

		public BigInteger getP()
		{
			return p;
		}

		public BigInteger getG()
		{
			return g;
		}

		public BigInteger getQ()
		{
			return q;
		}

		public int getJ()
		{
			return j;
		}

		public DHValidationParameters getValidationParameters()
		{
			return validation;
		}

		public override  bool Equals(Object  obj)
		{
			if (!(typeof(DHParameters).IsInstanceOfType(obj))) return false;
			DHParameters    pm = (DHParameters) obj;

			if (this.getValidationParameters() != null)
			{
				if (!this.getValidationParameters().Equals(pm.getValidationParameters()))
				{
					return false;
				}
			}
			else
			{
				if (pm.getValidationParameters() != null)
				{
					return false;
				}
			}

			if (this.getQ() != null)
			{
				if (!this.getQ().Equals(pm.getQ()))
				{
					return false;
				}
			}
			else
			{
				if (pm.getQ() != null)
				{
					return false;
				}
			}
        
			return (j == pm.getJ()) && pm.getP().Equals(p) && pm.getG().Equals(g);
		}
		public override int GetHashCode()
		{
			return base.GetHashCode();
		}

	}
}