using System;
using System.IO;
using System.Text;
using org.bouncycastle.crypto;
using org.bouncycastle.util.encoders;
using org.bouncycastle.util.test;

namespace org.bouncycastle.util.test
{
	/**
	 * a basic test that takes a cipher, key parameter, and an input
	 * and output string. This test wraps the engine in a buffered block
	 * cipher with padding disabled.
	 */
	public class BlockCipherVectorTest: Test
	{
		int                 id;
		BlockCipher         engine;
		CipherParameters    param;
		byte[]              input;
		byte[]              output;

		public BlockCipherVectorTest(
			int                 id,
			BlockCipher         engine,
			CipherParameters    param,
			String              input,
			String              output)
		{
			this.id = id;
			this.engine = engine;
			this.param = param;
			this.input = Hex.decode(input);
			this.output = Hex.decode(output);
		}

		public String getName()
		{
			return engine.getAlgorithmName() + " Vector Test " + id;
		}

		public TestResult perform()
		{
			BufferedBlockCipher cipher = new BufferedBlockCipher(engine);

			cipher.init(true, param);

			byte[]  outBytes = new byte[input.Length];

			int len1 = cipher.processBytes(input, 0, input.Length, outBytes, 0);
			try
			{
				cipher.doFinal(outBytes, len1);
			}
			catch (CryptoException e)
			{
				return new SimpleTestResult(false, 
					getName() + ": failed - exception " + e.ToString());
			}

			if (!isEqualArray(outBytes, output))
			{
				return new SimpleTestResult(false,
					getName() + ": failed - " + "expected " + 
					ByteArray2String(Hex.encode(output)) + 
					" got " + ByteArray2String(Hex.encode(outBytes)));
			}

			cipher.init(false, param);

			int len2 = cipher.processBytes(output, 0, output.Length, outBytes, 0);

			try
			{
				cipher.doFinal(outBytes, len2);
			}
			catch (CryptoException e)
			{
				return new SimpleTestResult(false, 
					getName() + ": failed reversal - exception " + e.ToString());
			}

			if (!isEqualArray(input, outBytes))
			{
//				return new SimpleTestResult(false, getName() + ": failed reversal got " + new String(Hex.encode(outBytes)));
				return new SimpleTestResult(false, getName() + ": failed reversal got " 
					+ ByteArray2String(Hex.encode(outBytes)));
			}

			return new SimpleTestResult(true, getName() + ": OKAY");
		}

		private bool isEqualArray(
			byte[]  a,
			byte[]  b)
		{
			if (a.Length != b.Length)
			{
				return false;
			}

			for (int i = 0; i != a.Length; i++)
			{
				if (a[i] != b[i])
				{
					return false;
				}
			}

			return true;
		}

		protected String ByteArray2String(byte[] bs)
		{
			StringBuilder s = new StringBuilder();
			for (int i = 0; i < bs.Length; i++) 
			{
				s.Append(Convert.ToChar(bs[i]));
			}
			return s.ToString();
		}
	}
}
