using System;
using System.Text;
//using org.bouncycastle.math;
//using org.bouncycastle.math.ec;
//using org.bouncycastle.security;
using org.bouncycastle.crypto;
//using org.bouncycastle.crypto.signers;
using org.bouncycastle.crypto.digests;
//using org.bouncycastle.crypto.engines;
//using org.bouncycastle.crypto.agreement;
//using org.bouncycastle.crypto.generators;
using org.bouncycastle.crypto.parameters;
using org.bouncycastle.crypto.macs;
//using org.bouncycastle.crypto.modes;
//using org.bouncycastle.crypto.paddings;
using org.bouncycastle.util.encoders;
using org.bouncycastle.util.test;


namespace test.crypto
{
	/**
	 * MD5 HMac Test, test vectors from RFC 2202
	 */
	public class MD5HMacTest: Test
	{
		readonly static String[] keys = 
		{
			"0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b",
			"4a656665",
			"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
			"0102030405060708090a0b0c0d0e0f10111213141516171819",
			"0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c",
			"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
			"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
		};

		readonly static String[] digests = {
			"9294727a3638bb1c13f48ef8158bfc9d",
			"750c783e6ab0b503eaa86e310a5db738",
			"56be34521d144c88dbb8c733f0e8b3f6",
			"697eaf0aca3a3aea3a75164746ffaa79",
			"56461ef2342edc00f9bab995690efd4c",
			"6b1ab7fe4bd7bf8f0b62e6ce61b9d0cd",
			"6f630fad67cda0ee1fb1f562db3aa53e"
		};

		readonly static String[] messages = {
			"Hi There",
			"what do ya want for nothing?",
			"0xdddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddd",
			"0xcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcd",
			"Test With Truncation",
			"Test Using Larger Than Block-Size Key - Hash Key First",
			"Test Using Larger Than Block-Size Key and Larger Than One Block-Size Data"
		};

		public static bool arraysEqual(byte[] a, byte[] b)
		{
			if (a == b) return true;
			if (a.Length != b.Length) return false;

			for (int i = 0; i < a.Length; i++)
			{
				if (a[i] != b[i]) return false;
			}

			return true;
		}
		
		public String getName()
		{
			return "MD5HMac";
		}

		public TestResult perform()
		{
			HMac hmac = new HMac(new MD5Digest());
			byte[] resBuf = new byte[hmac.getMacSize()];

			for (int i = 0; i < messages.Length; i++)
			{
				byte[] m = getBytes(messages[i]);
				if (messages[i].StartsWith("0x"))
				{
					m = Hex.decode(messages[i].Substring(2));
				}
				hmac.init(new KeyParameter(Hex.decode(keys[i])));
				hmac.update(m, 0, m.Length);
				hmac.doFinal(resBuf, 0);

				if (!arraysEqual(resBuf, Hex.decode(digests[i])))
				{
					return new SimpleTestResult(false, getName() + "Vector " + i + " failed");
				}
			}

			// test reset
			int vector = 0; // vector used for test
			byte[] mm = getBytes(messages[vector]);
			if (messages[vector].StartsWith("0x"))
			{
				mm = Hex.decode(messages[vector].Substring(2));
			}
			hmac.init(new KeyParameter(Hex.decode(keys[vector])));
			hmac.update(mm, 0, mm.Length);
			hmac.doFinal(resBuf, 0);
			hmac.reset();
			hmac.update(mm, 0, mm.Length);
			hmac.doFinal(resBuf, 0);

			if (!arraysEqual(resBuf, Hex.decode(digests[vector])))
			{
				return new SimpleTestResult(false, getName() +
					"Reset with vector " + vector + " failed");
			}

			return new SimpleTestResult(true, getName() + ": Okay");
		}
		private byte [] getBytes(String str)
		{
			char[]  cs = str.ToCharArray();
			byte[]  bs = new byte[cs.Length];

			for (int i = 0; i != cs.Length; i++)
			{
				bs[i] = Convert.ToByte(cs[i]);
			}
			return bs;
		}

		[STAThread]
		static void Main(string[] args)
		{
			MD5HMacTest   test = new MD5HMacTest();
			TestResult    result = test.perform();

			Console.WriteLine(result.ToString());
		}
	}

}
