using System;
using System.Text;
//using org.bouncycastle.math;
//using org.bouncycastle.math.ec;
//using org.bouncycastle.security;
using org.bouncycastle.crypto;
//using org.bouncycastle.crypto.signers;
using org.bouncycastle.crypto.digests;
//using org.bouncycastle.crypto.engines;
//using org.bouncycastle.crypto.agreement;
//using org.bouncycastle.crypto.generators;
using org.bouncycastle.crypto.parameters;
using org.bouncycastle.crypto.macs;
//using org.bouncycastle.crypto.modes;
//using org.bouncycastle.crypto.paddings;
using org.bouncycastle.util.encoders;
using org.bouncycastle.util.test;


namespace test.crypto
{
	/**
	 * RIPEMD160 HMac Test, test vectors from RFC 2286
	 */
	public class RIPEMD160HMacTest: Test
	{
		readonly static String[] keys = 
		{
			"0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b",
			"4a656665",
			"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
			"0102030405060708090a0b0c0d0e0f10111213141516171819",
			"0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c",
			"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
			"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
		};

		readonly static String[] digests = {
			"24cb4bd67d20fc1a5d2ed7732dcc39377f0a5668",
			"dda6c0213a485a9e24f4742064a7f033b43c4069",
			"b0b105360de759960ab4f35298e116e295d8e7c1",
			"d5ca862f4d21d5e610e18b4cf1beb97a4365ecf4",
			"7619693978f91d90539ae786500ff3d8e0518e39",
			"6466ca07ac5eac29e1bd523e5ada7605b791fd8b",
			"69ea60798d71616cce5fd0871e23754cd75d5a0a"
		};

		readonly static String[] messages = {
			"Hi There",
			"what do ya want for nothing?",
			"0xdddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddd",
			"0xcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcd",
			"Test With Truncation",
			"Test Using Larger Than Block-Size Key - Hash Key First",
			"Test Using Larger Than Block-Size Key and Larger Than One Block-Size Data"
		};

		public static bool arraysEqual(byte[] a, byte[] b)
		{
			if (a == b) return true;
			if (a.Length != b.Length) return false;

			for (int i = 0; i < a.Length; i++)
			{
				if (a[i] != b[i]) return false;
			}

			return true;
		}
        
		public String getName()
		{
			return "RIPEMD160HMac";
		}

		public TestResult perform()
		{
			HMac hmac = new HMac(new RIPEMD160Digest());
			byte[] resBuf = new byte[hmac.getMacSize()];

			for (int i = 0; i < messages.Length; i++)
			{
				byte[] m = getBytes(messages[i]);
				if (messages[i].StartsWith("0x"))
				{
					m = Hex.decode(messages[i].Substring(2));
				}
				hmac.init(new KeyParameter(Hex.decode(keys[i])));
				hmac.update(m, 0, m.Length);
				hmac.doFinal(resBuf, 0);

				if (!arraysEqual(resBuf, Hex.decode(digests[i])))
				{
					return new SimpleTestResult(false, getName() + ": Vector " + i + " failed");
				}
			}

			return new SimpleTestResult(true, getName() + ": Okay");
		}
		private byte [] getBytes(String str)
		{
			char[]  cs = str.ToCharArray();
			byte[]  bs = new byte[cs.Length];

			for (int i = 0; i != cs.Length; i++)
			{
				bs[i] = Convert.ToByte(cs[i]);
			}
			return bs;
		}
		private String ByteArray2String(byte[] bs)
		{
			StringBuilder s = new StringBuilder();
			for (int i = 0; i < bs.Length; i++) 
			{
				s.Append(Convert.ToChar(bs[i]));
			}
			return s.ToString();
		}
		[STAThread]
		static void Main(string[] args)
		{
			RIPEMD160HMacTest   test = new RIPEMD160HMacTest();
			TestResult          result = test.perform();

			Console.WriteLine(result.ToString());
		}
	}

}
