using System;
using System.Text;
//using org.bouncycastle.math;
//using org.bouncycastle.math.ec;
//using org.bouncycastle.security;
using org.bouncycastle.crypto;
//using org.bouncycastle.crypto.signers;
using org.bouncycastle.crypto.digests;
//using org.bouncycastle.crypto.engines;
//using org.bouncycastle.crypto.encodings;
//using org.bouncycastle.crypto.generators;
using org.bouncycastle.crypto.parameters;
using org.bouncycastle.crypto.macs;
//using org.bouncycastle.crypto.modes;
//using org.bouncycastle.crypto.paddings;
using org.bouncycastle.util.encoders;
using org.bouncycastle.util.test;

namespace test.crypto
{
	/**
	 * SHA1 HMac Test, test vectors from RFC 2202
	 */
	public class SHA1HMacTest: Test
	{
		readonly static String[] keys = 
		{
			"0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b",
			"4a656665",
			"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
			"0102030405060708090a0b0c0d0e0f10111213141516171819",
			"0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c",
			"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
			"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
		};

		readonly static String[] digests = {
			"b617318655057264e28bc0b6fb378c8ef146be00",
			"effcdf6ae5eb2fa2d27416d5f184df9c259a7c79",
			"125d7342b9ac11cd91a39af48aa17b4f63f175d3",
			"4c9007f4026250c6bc8414f9bf50c86c2d7235da",
			"4c1a03424b55e07fe7f27be1d58bb9324a9a5a04",
			"aa4ae5e15272d00e95705637ce8a3b55ed402112",
			"e8e99d0f45237d786d6bbaa7965c7808bbff1a91",
			"4c1a03424b55e07fe7f27be1d58bb9324a9a5a04",
			"aa4ae5e15272d00e95705637ce8a3b55ed402112",
			"e8e99d0f45237d786d6bbaa7965c7808bbff1a91"
		};

		readonly static String[] messages = {
			"Hi There",
			"what do ya want for nothing?",
			"0xdddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddd",
			"0xcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcd",
			"Test With Truncation",
			"Test Using Larger Than Block-Size Key - Hash Key First",
			"Test Using Larger Than Block-Size Key and Larger Than One Block-Size Data"
		};

		public static bool arraysEqual(byte[] a, byte[] b)
		{
			if (a == b) return true;
			if (a.Length != b.Length) return false;

			for (int i = 0; i < a.Length; i++)
			{
				if (a[i] != b[i]) return false;
			}

			return true;
		}
        
		public String getName()
		{
			return "SHA1HMac";
		}

		public TestResult perform()
		{
			HMac hmac = new HMac(new SHA1Digest());
			byte[] resBuf = new byte[hmac.getMacSize()];

			for (int i = 0; i < messages.Length; i++)
			{
				byte[] m = getBytes(messages[i]);
				if (messages[i].StartsWith("0x"))
				{
					m = Hex.decode(messages[i].Substring(2));
				}
				hmac.init(new KeyParameter(Hex.decode(keys[i])));
				hmac.update(m, 0, m.Length);
				hmac.doFinal(resBuf, 0);

				if (!arraysEqual(resBuf, Hex.decode(digests[i])))
				{
					return new SimpleTestResult(false, getName() + ": Vector " + i + " failed");
				}
			}

			//
			// test reset
			//
			int vector = 0; // vector used for test
			byte[] mm = getBytes(messages[vector]);
			if (messages[vector].StartsWith("0x"))
			{
				mm = Hex.decode(messages[vector].Substring(2));
			}
			hmac.init(new KeyParameter(Hex.decode(keys[vector])));
			hmac.update(mm, 0, mm.Length);
			hmac.doFinal(resBuf, 0);
			hmac.reset();
			hmac.update(mm, 0, mm.Length);
			hmac.doFinal(resBuf, 0);

			if (!arraysEqual(resBuf, Hex.decode(digests[vector])))
			{
				return new SimpleTestResult(false, getName() +
					"Reset with vector " + vector + " failed");
			}

			return new SimpleTestResult(true, getName() + ": Okay");
		}
		private byte [] getBytes(String str)
		{
			char[]  cs = str.ToCharArray();
			byte[]  bs = new byte[cs.Length];

			for (int i = 0; i != cs.Length; i++)
			{
				bs[i] = Convert.ToByte(cs[i]);
			}
			return bs;
		}
		private String ByteArray2String(byte[] bs)
		{
			StringBuilder s = new StringBuilder();
			for (int i = 0; i < bs.Length; i++) 
			{
				s.Append(Convert.ToChar(bs[i]));
			}
			return s.ToString();
		}



		[STAThread]
		static void Main(string[] args)
		{
			SHA1HMacTest    test = new SHA1HMacTest();
			TestResult      result = test.perform();

			Console.WriteLine(result.ToString());
		}
	}
}
